import { Utils, Bluetooth as S4Bluetooth } from "syrus4-nodejs"

export class Bluetooth {
	constructor({ name, host }) {
		this.name = name
		this.host = host
		this.active = true

		this.onConnect = null
		this.onDisconnect = null
		this.onMessage = null

		this.characteristic = this.host == 'apps' ? 'user_apps_console' : 'events'
		this.devices = new Set()
		this.interval = null
		this.init()
	}

	get disable_welcome_message() { return true }
	get no_disconnects() { return true }

	async init() {
		// subscribe to notifications to monitor [dis]connections
		await this.scan_devices()
		S4Bluetooth.onBluetoothUpdate((channel, payload) => {
			if (!channel.endsWith('/update')) return
			console.log(`bt(${this.name}) watcher`, {channel, payload})
			let { mac, connected, authenticated, characteristics } = payload
			let enabled = connected && authenticated
			if (enabled && characteristics.includes(this.characteristic)) {
				this.devices.add(mac)
				return
			}
			this.devices.delete(mac)
		})
		if (this.characteristic == 'user_apps_console') {
			S4Bluetooth.onAppConsoleMessage((message) => {
				console.log(`bt(${this.name}):rx`, message)
				if (this.onMessage) this.onMessage(message)
			})
		}
		// set an interval to fetch the state of the devices
		this.interval = setInterval(this.scan_devices, 1000 * 60 * 10);
	}
	async scan_devices() {
		let response
		try {
			response = await Utils.OSExecute('apx-bt ble_status')
		} catch(error) {
			console.error(error)
			clearInterval(this.interval)
			return
		}
		let { ble_devices } = response
		let devices = new Set()
		for (const device in ble_devices) {
			let {mac, connected, authenticated, characteristics} = device
			let enabled = connected && authenticated
			if (enabled && characteristics.includes(this.characteristic)){
				devices.add(mac)
			}
		}
		this.devices = devices
	}

	//TODO: properly support these when we receive client connnect/disconnect signals from apex
	connect() { if (this.onConnect) this.onConnect() }
	disconnect() { }

	enable() {
		if (!this.active) {
			console.log(`bt(${this.name}): enabled`);
			this.active = true;
			this.connect()
		}
	}
	disable() {
		if (this.active) {
			console.log(`bt(${this.name}): disabled`);
			this.active = false;
		}
	}
	destroy() {}

	async send(data, next) {
		if (!next) { next = () => {} }
		if (!this.active) return
		if (!this.devices.size) {
			next(`bt(${this.name}): ERROR: no devices connected`)
			return
		}
		if(typeof data == 'object'){
			data = JSON.stringify(data)
		}
		try{
			let bt_console = this.characteristic
			let command = `apx-bt send ${bt_console} '${data}'`
			console.log(`bt(${this.name}):tx`, data)
			await Utils.OSExecute(command)
			next()
		} catch(error){
			next(error)
		}
		return
	}
}
