import _intersection from "lodash.intersection"

import { Network } from "syrus4-nodejs"
import { Queue } from "../services/Queue.js"

/**
 * Abstract class where all the channels must be inherit
 */
export class Channel {
	constructor({ name, host, path, port, allowed_interface = "*", allowed_ssid = "*", allowed_ip = "*" }) {
		this.onConnect = null
		this.onDisconnect = null
		this.onMessage = null
		this.path = host || path
		this.port = port
		this.name = name
		this.active = true
		this.network_config = { allowed_interface, allowed_ssid, allowed_ip }
		// Queue for storage events
		this.queue = new Queue(`peg_queue_${name}`)
		this.ready = new Promise((resolve, reject) => {
			this.resolve = resolve
			this.init()
		});
		Network.onNetworkChange(() => { this.checkNetworks(this.network_config) })
	}
	async init(){
		await this.queue.ready().catch(console.error)
		await this.checkNetworks(this.network_config).catch(console.error)
		this.resolve()
	}
	/**
	 * Check the network state to enable/disable destination based on config
	 * Ej only allow wifi :define destination local json mqtt://192.168.1.116:1883 protocol="mqtt" allowed_interace=wifi
	 *
	 * @param {*} allowed_interface
	 * @param {*} allowed_ssid
	 * @param {*} allowed_ip
	 */
	async checkNetworks({ allowed_interface = "*", allowed_ssid = "*", allowed_ip = "*" }) {
		let allowed = true,
			names = [],
			ips = [],
			ssids = []
		if (allowed_interface != "*" || allowed_ssid != "*" || allowed_ip != "*") {
			let networks = await Network.getNetworks();
			for (const net in networks) {
				const network = networks[net];
				let netname = net;
				switch (net) {
					case "wlan0":
						netname = "wifi";
						break;
					case "eth0":
						netname = "eth";
						break;
					case "ppp0":
						netname = "cell";
						break;
					default:
						break;
				}
				if (network.connected) {
					names.push(netname);
					ips.push(network.ip_address);
					ssids.push(network.ssid);
				}
			}
			if (allowed_interface != "*") {
				if (_intersection(names, allowed_interface.split(",")).length == 0) allowed = false;
			}
			if (allowed_ip != "*") {
				if (_intersection(ips, allowed_ip.split(",")).length == 0) allowed = false;
			}
			if (allowed_ssid != "*") {
				if (_intersection(ssids, allowed_ssid.split(",")).length == 0) allowed = false;
			}
		}
		if (allowed) {
			this.enable();
		} else {
			this.disable();
		}
	}

	/**
	 * @abstract Method for destoy the destination
	 */
	destroy() {}

	/**
	 * @abstract Method for enable the destination
	 */
	enable() {
		if (!this.active) {
			console.log("activating destination ", this.name);
			this.active = true;
			this.connect(false);
		}
	}

	/**
	 * @abstract Method for disable the destination
	 */
	disable() {
		if (this.active) {
			console.log("deactivating destination", this.name);
			this.active = false;
			this.disconnect(true);
		}
	}

	/**
	 * @abstract Method for connect the destination
	 */
	connect(force = true) {}

	/**
	 * @abstract Method for disconnect the destination
	 */
	disconnect(force = true) {}

	/**
	 * @abstract Method for send data to the destination
	 * also must manage the queue
	 */
	send(data, next) {}
}
