import mqtt from "mqtt"
import { Channel } from "./Channel.js"
const defaults = {
	port: "1883",
	subscribe: "esp/messagesrx",
	publish: "esp/messagestx",
	autoDequeue: true,
	reconnectPeriod: 5000,
	keepalive: 10
};
export class MQTT extends Channel {
	constructor(config) {
		config = Object.assign({}, defaults, config);
		super(config);
		this.name = config.name;
		this.host = config.host;
		this.port = config.port;
		this.onCommand = config.onCommand;
		this.clientId = config.clientId;
		this.username = config.username;
		this.password = config.password;
		this.protocol = config.protocolId;
		this.commands_sub = config.commands_sub || config.subscribe;
		this.commands_pub = config.commands_pub;
		this.publish = config.publish;
		this.keepalive = config.keepalive;
		this.autoDequeue = config.autoDequeue;
		this.reconnectPeriod = config.reconnectPeriod || 5000;
		this.ack = config.ack;

		if (!this.protocol) {
			switch (config.port) {
				case "1883":
					this.protocol = "mqtt";
					break;
				case "8883":
					this.protocol = "mqtts";
					break;
				case "8080":
				case "80":
					this.protocol = "ws";
					break;
				case "443":
				case "8081":
					this.protocol = "wss";
					break;
				default:
					this.protocol = "mqtt";
					break;
			}
		}
		this.config = {
			clientId: this.clientId,
			username: this.username,
			password: this.password,
			protocol: this.protocol,
			host: this.host,
			hostname: this.host,
			keepalive: this.keepalive,
			reconnectPeriod: this.reconnectPeriod,
			port: this.port,
			clean: config.clean != null ? config.clean : true
		};
		if (config.lastWillTopic && config.lastWillPayload) {
			this.config.will = {
				topic: config.lastWillTopic,
				qos: config.lastWillQoS,
				payload: config.lastWillPayload,
				retain: config.lastWillRetain
			};
		}
	}

	destroy() {
		this.client.end(false, () => {
			this.client.removeAllListeners();
			this.client = null;
		});
	}

	async connect() {
		await this.ready;
		if (!this.active) return false;
		if (!this.client) {
			this.client = mqtt.connect(this.config);
			this.client.on("connect", () => {
				console.log("MQTT Client: connection established with server", this.host);
				if (this.onConnect) this.onConnect();
				this._sendBuffer();
			});
			if (this.commands_sub) {
				this.client.subscribe(this.commands_sub, { qos: 1 }, (err, message) => {
					console.log("connected to subscribe");
				});
			}
			this.client.on("message", async (topic, message) => {
				console.log("message received, topic", topic, "message", message.toString());
				var data = message.toString();
				try {
					data = JSON.parse(data);
					if (this.onCommand) var response = await this.onCommand({ command: data.command, isSyrusLang: true });
				} catch (error) {
					if (this.onCommand) response = await this.onCommand({ command: data, isSyrusLang: true });
				}
				console.log(response);
				this.client.publish(this.commands_pub || "dev/commands_pub", `${response}`, { qos: 1, retain: true });
			});
			this.client.on("error", err => {
				console.error(err);
				if (this.onError) this.onError(err);
			});
			this.client.on("disconnect", err => {
				console.error(err);
			});
			this.client.on("close", err => {
				console.error("mqtt on close", err);
			});
			this.client.on("reconnect", () => {
				console.log("reconnected to", this.host);
				this._sendBuffer();
			});
		} else {
			this.client.reconnect();
		}
	}

	disconnect() {
		if (this.client) this.client.end();
		if (this.onDisconnect) this.onDisconnect();
	}

	send(payload, opts) {
		this.queue.push(JSON.stringify(payload));
		if (this.queue.length == 1 && this.active) {
			this._sendBuffer(opts);
		}
	}

	_sendBuffer(opts = { ack: "seq", retain: true }) {
		if (this.queue.length == 0) {
			return;
		}
		var payload = this.queue[0];
		this.client.publish(
			this.publish,
			payload,
			{
				qos: opts.ack == "none" ? 0 : 2,
				retain: opts.retain
			},
			err => {
				if (!err) {
					console.log("MQTT message transmitted", payload);
					this.queue.shift();
					return this._sendBuffer(opts);
				} else {
					console.error("MQTT err", err);
				}
			}
		);
	}
}
