import net from "net"
import { Channel } from "./Channel.js"

export class TCP extends Channel {
	constructor(config) {
		super(config);
		this.config = {};
		this.onConnect = null;
		this.onDisconnect = null;
		this.onMessage = null;
		this.reconnectTime = 5 * 1000;
		this.keepAliveTime = 1000 * 60 * 7;
		this.autoReconect = true;
		this.keepAlive = config.keepAliveMessage;
		this.host = config.host;
		this.port = config.port;
		this.client = new net.Socket();
		this.client.setKeepAlive(true, 7 * 60 * 1000);

		this.client.on("connect", async () => {
			console.log("TCP Client: connection established with server");
			this.reconnectTime = 5 * 1000;
			this.resetKeepAliveInterval();
			if (this.onConnect) this.onConnect();
		});

		this.client.on("close", reason => {
			clearInterval(this.keepAlive);
			if (this.onDisconnect) this.onDisconnect();
			// Restart connection in x secs
			if (this.autoReconect) {
				if (this.active) {
					this.reconnectTime *= 2;
					if (this.reconnectTime > 300 * 1000) {
						this.reconnectTime = 300 * 1000;
					}
					console.log(`Connection closed, reconnection in ${this.reconnectTime / 1000} seconds`);
				}
				setTimeout(() => {
					this.connect();
				}, this.reconnectTime);
			}
		});

		this.client.on("data", data => {
			data = data.toString()
			console.log(`tcp(${this.name}):rx`, data.trim())
			if (this.onMessage) { this.onMessage(data) }
		});

		this.client.on("error", function(err) {
			console.log("error on TCP connection", arguments);
			if (this.onError) {
				this.onError(arguments);
			}
		});
	}

	destroy() {
		this.client.autoReconect = false;
		this.client.removeAllListeners();
		this.client.end(() => {
			this.client.destroy();
			this.client = null;
		});
	}

	connect() {
		this.ready.then(() => {
			if (!this.active) return false;
			this.client.connect({
				host: this.host,
				port: this.port
			});
		});
	}

	disconnect() {
		this.client.destroy();
	}

	send(data, next) {
		if (!this.active) return false;
		console.info(`tcp(${this.name}):tx`, data);
		this.client.write(data, next);
		this.resetKeepAliveInterval();
	}

	resetKeepAliveInterval() {
		clearInterval(this.keepAlive);
		this.keepAlive = setInterval(() => {
			console.log("send keep alive", this.keepAliveMessage);
			this.client.write(this.keepAliveMessage);
		}, this.keepAliveTime);
	}
}
