import storage from "../../services/storage.js"

import { Utils } from "syrus4-nodejs"

const VIDEO_ERROR_STATES = {
	'CAMERA_NOT_AVAILABLE': 5,
	'MEMORY_ERROR': 14,
	'ERROR_CREATING_CLIP': 15,
}
export const MediaManagerMixin = Base => class extends Base {
	getMediaState(key) {
		return this.media_state[key]
	}
	// Photo
	async getPS(fields, data, opts) {
		// console.log({fields, data, opts});
		// console.log('media_state', this.media_state)
		// return ""
		if (!this.ftp.username) {
			console.log('MEDIA_ERROR: media requested without FTP setup')
			return ""
		}
		let tag = ""
		const { photo, video } = opts
		if (photo) tag += await this.uploadPhoto(data, opts)
		if (video) tag += await this.uploadVideos(data, opts)
		return tag
	}

	async updateMediaState(key, value) {
		this.media_state[key] = value
		await storage.hset(`${this.name}_media_state`, key, value)
	}

	async uploadPhoto(data, opts) {
		let tag = ""
		let source = opts.photo
		let key
		if (source.startsWith('fatigue_sensor')) source = `$${source}`
		if (!source.startsWith('$fatigue_sensor')) return ""
		key = "$fatigue_sensor.latest_photo"
		if (source.includes('.')) {
			key = `$fatigue_sensor.photos.${source.split('.')[1]}`
		}
		let photo_src = data[key]
		if (!photo_src) return ""
		let photo_epoch = photo_src.split('-')[0]
		let last = this.getMediaState(key)
		if (photo_epoch <= last) return ""
		if (source.startsWith('fatigue_sensor')) {
			photo_src = `/data/users/syrus4g/fatigue_sensor/${photo_src}`
		}
		let photo_type = "00"
		let photo_id = parseInt(Date.now() / 1e3) % 1e8
		let photo_dest = `${this.imei}/images/${photo_id}.jpeg`
		this.updateMediaState(key, photo_epoch)
		this.queueMedia(photo_id, photo_type, photo_src, photo_dest, false)
		tag += `;PS${photo_type}=${photo_id}`
		return tag
	}

	async uploadVideos(data, opts) {
		let tag = ""
		let media_type = 0
		let media_id = (parseInt(Date.now() / 1e3) % 1e8) * 10
		const sources = opts.video.split(',')
		for (const _source of sources) {
			const source = data[_source] || _source
			const video_media = `V${media_type}`
			this.waitForVideo(source, video_media, media_id, opts)
			tag += `;PS${video_media}=${media_id}`
			media_type++
			media_id++
		}
		return tag
	}

	async waitForVideo(source, media_type, media_id, opts, attempt = 0) {
		attempt++
		let timeout = 10 * 1000
		let cmd = `apx-video clip_status --name=${source}`
		if (opts.camera) cmd += ` --camera=${opts.camera}`
		const videos = await Utils.OSExecute(cmd)
		const video = videos[0] || { system_epoch: Date.now()/1e3, event: source, camera: opts.camera, file: {state: 'NOT_READY'} }
		console.log({video, attempt})
		let error_code = VIDEO_ERROR_STATES[video.file.state]
		if (error_code) {
			this.priorityPush(this.msgbuilder(`RXASCN${error_code},${media_id},${media_type}`))
			return
		}
		if (video.file.state == "CLIP_QUEUED") {
			timeout = 20 * 1000
			attempt = 0
		}
		if (video.file.state != "CLIP_READY") {
			if (attempt == 12) {
				error_code = 10
				this.priorityPush(this.msgbuilder(`RXASCN${error_code},${media_id},${media_type}`))
				console.log(`VIDEO_ERROR: ${source} was never resolved`)
				return
			}
			await Utils.$sleep(timeout)
			return await this.waitForVideo(source, media_type, media_id, opts, attempt)
		}
		// multiple videos not supported yet
		const { system_epoch, camera, file } = video
		if (opts.camera && camera != opts.camera) return
		const key = `${source}-${camera}`
		const media_state = this.getMediaState(key)
		if (system_epoch <= media_state) return
		const media_dest = `${this.imei}/images/${media_id}.mp4`
		console.log({ media_id, media_type, path: file.path, media_dest, delay_upload: opts.delay_upload })
		await this.updateMediaState(key, system_epoch)
		await this.queueMedia(media_id, media_type, file.path, media_dest, opts.delay_upload)
	}

	/**
	 * Queue photo for upload to FTP
	 * @param {*} photo_id
	 * @param {*} filename
	 * @memberof TAIP
	 */
	async queueMedia(media_id, media_type, media_src, media_dest, delay_upload = false) {
		// store in hash based on media_id, for uploading later
		const data = `${media_id}::${media_type}::${media_src}::${media_dest}`
		const requested = await storage.hget(`${this.name}_media_requested`, media_id)
		if (delay_upload && !requested) {
			await storage.hset(`${this.name}_media`, media_id, data)
			return
		}
		// push media to key
		await storage.push(`${this.name}_media_queue`, data)
		this.sendMedia()
	}
	/**
	 *
	 * Upload Media, add to queue
	 * @param {*} media_id
	 * @memberof TAIP
	 */
	async uploadMedia(media_id) {
		let data = await storage.hget(`${this.name}_media`, media_id)
		await storage.hdel(`${this.name}_media`, media_id)
		await storage.push(`${this.name}_media_queue`, data)
	}
	/**
	 * uploads photo, sends respective SXACN commands
	 * @memberof TAIP
	 */
	async sendMedia() {
		let queue = `${this.name}_media_queue`
		let media_list = await storage.slice(queue, -1, -1)
		if (!media_list.length) return
		let media = media_list[0]
		let [media_id, media_type, src, dest] = media.split('::')
		try {
			this.priorityPush(this.msgbuilder(`RXASCN3,${media_id},${media_type}`))
			let cmd = ""
			cmd += `lftp -u ${this.ftp.username},${this.ftp.password} -p ${this.ftp.port}`
			cmd += ` -e 'put ${src} -o ${dest}; bye'`
			cmd += ` ftp://${this.ftp.server}`
			console.log(cmd)
			await Utils.OSExecute(cmd)
			this.priorityPush(this.msgbuilder(`RXASCN4,${media_id},${media_type}`))
		} catch (error) {
			console.error(error)
		}
		await storage.pop(queue)
		this.sendMedia()
		return
	}
}
