// handlers for the watchers, to check changes on S4, the
import fs from "fs"
import dotted from "dot-object"

import { GPS, IOS, System, Network} from "syrus4-nodejs"
import { Accelerometer, ECU, Geofences } from "syrus4-nodejs"
import { Bluetooth } from "syrus4-nodejs"
import { IButton, Temperature } from "syrus4-nodejs"
import { Serial } from "syrus4-nodejs"
import { Fatigue } from "syrus4-nodejs"

import dayjs from "../utils/dayjs.js"
import { syrusjs_variables, fireInternalSignal } from "./globals.js"

// sets all keys on an object as null
function clear_object(object){
	for (const key in object) {
		object[key] = null
	}
}
function isObject(object){
	return Object.prototype.toString.call(object) === '[object Object]'
}
// TODO: don't initialize all watchers unless required,
// this is a sideffect from favoring TAIP, that requires certain watchers always running
export function startWatchers(configuration) {
	function onGpsData(gps) {
		Object.assign(syrusjs_variables, dotted.dot({ $gnss: gps.coords }));
		syrusjs_variables["$gnss.heading"] = gps.coords.bearing;
		syrusjs_variables["$gnss.mph"] = gps.coords.speed * 2.23694;
		syrusjs_variables["$gnss.kph"] = gps.coords.speed * 3.6;
		syrusjs_variables["$gnss.timestamp"] = gps.timestamp;

		syrusjs_variables["$speed"] = Math.round(gps.coords.speed);
		syrusjs_variables["$mph"] = Math.round(gps.coords.speed * 2.23694);
		syrusjs_variables["$kph"] = Math.round(gps.coords.speed * 3.6);

		let time = dayjs.unix(gps.timestamp)
		syrusjs_variables["$timestamp"] = time.locale("en").format("LLLL");
		syrusjs_variables["$timestamp.es"] = time.locale("es").format("LLLL");
		syrusjs_variables["$timestamp.fr"] = time.locale("fr").format("LLLL");
		syrusjs_variables["$timestamp.pt"] = time.locale("pt").format("LLLL");
		Object.assign(syrusjs_variables, dotted.dot({ $gnss: gps.extras }));
	}
	function onGeofenceChange(fences) {
		if (!Array.isArray(fences)) {
			fences = [fences];
		}

		for (const fence of fences) {
			syrusjs_variables[`$geofences.${fence.name}.inside`] = fence.is_inside;
			syrusjs_variables[`$geofences.${fence.name}.time`] = fence.time || fence.timestamp;

			syrusjs_variables[`@last_geofence.name`] = fence.name;
			syrusjs_variables[`@last_geofence.state`] = fence.is_inside ? "inside" : "outside";
			syrusjs_variables[`@last_geofence.time`] = fence.time || fence.timestamp;
			fireInternalSignal(`@geofence.${fence.is_inside ? "enter" : "exit"}`);
		}
	}
	function onGeofenceGroupChange(groups) {
		if (!Array.isArray(groups)) {
			groups = [groups];
		}

		for (const group of groups) {
			syrusjs_variables[`$geofences.$groups.${group.name}.inside`] = group.is_inside;
			syrusjs_variables[`$geofences.$groups.${group.name}.time`] = group.time || group.timestamp;

			syrusjs_variables[`@last_geofence.group.name`] = group.name;
			syrusjs_variables[`@last_geofence.group.state`] = group.is_inside ? "inside" : "outside";
			syrusjs_variables[`@last_geofence.group.time`] = group.time;
			fireInternalSignal(`@geofence.group.${group.is_inside ? "enter" : "exit"}`);
		}
	}
	function onNetworkChange() {
		Network.getNetworks()
			.then(networks => {
				for (const net in networks) {
					const network = networks[net];
					var netname = net;
					switch (net) {
						case "wlan0":
							netname = "wifi";
							break;
						case "eth0":
							netname = "eth";
							break;
						case "ppp0":
							netname = "cell";
							break;
						default:
							break;
					}
					for (const key in network) {
						syrusjs_variables[`$net_${netname}.${key}`] = network[`${key}`];
					}
				}
			})
			.catch(console.error);
	}
	function onInputChange(ios) {
		console.log('ios_change', ios)
		const signals = []
		for (const io in ios) {
			const io_lc = io.toLowerCase()
			const value = ios[io]
			if (io == "BAT") {
				onBatteryChange(Number(value))
			} else {
				syrusjs_variables[`$io.${io_lc}`] = value
				if (value) signals.push(`@io.${io_lc}`)
			}
		}
		fireInternalSignal(...signals)
	}
	function onBatteryChange(vdc) {
		/**
		 * battery level between 4000 and 3500
		 * keys: "$battery.level" , "$battery.voltaje"
		 * bat full 4000 >= 100%
		 * bat low  3620 <= 20%
		 * bat cri  3560 <= 10%
		 */
		if (vdc == null) {
			syrusjs_variables[`$battery.connected`] = false;
			syrusjs_variables[`$battery.voltage`] = 0;
			syrusjs_variables[`$battery.mv`] = 0;
			syrusjs_variables[`$battery.level`] = 0;
			return;
		}
		syrusjs_variables[`$battery.connected`] = true;
		syrusjs_variables[`$battery.voltage`] = vdc;
		syrusjs_variables[`$battery.mv`] = Math.round(vdc * 1000);
		var level = Math.round(((vdc * 1000 - 3500) / 500) * 100);
		if (level > 100) level = 100;
		if (level < 0) level = 0;
		if (syrusjs_variables[`$battery.level`] < 100 && level >= 100) {
			fireInternalSignal("@battery.full");
		}
		if (syrusjs_variables[`$battery.level`] > 20 && level <= 20) {
			fireInternalSignal("@battery.low");
		}
		if (syrusjs_variables[`$battery.level`] > 10 && level <= 10) {
			fireInternalSignal("@battery.critical");
		}
		syrusjs_variables[`$battery.level`] = level;
	}
	function onMotionChange(isMoving) {
		syrusjs_variables["$accelerometer.motion"] = isMoving;
	}
	function onMotionEvent(event, value, time = null) {
		syrusjs_variables[`$accelerometer.${event}.time`] = time ? time : new Date().toISOString();
		syrusjs_variables[`$accelerometer.${event}.value`] = value;
		fireInternalSignal(`@accelerometer.${event}.signal`);
	}
	const tires_dotted = {}
	function onEcuParamsChange(ecu_params) {
		const flat_data = {}
		const signals = []
		for (const key in ecu_params) {
			let value = ecu_params[key]
			if (key.startsWith('@')){
				signals.push(key)
				continue
			}
			if (key.startsWith('$')){
				flat_data[key] = value
			}
			if (isObject(value)){
				let obj = {}
				obj[key] = value
				let flat = dotted.dot(obj)
				for(const fkey in flat){
					syrusjs_variables[`$ecu.${fkey}`] = flat[fkey]
				}
				continue
			}
			syrusjs_variables[`$ecu.${key}`] = value
		}
		const sub_data = dotted.object(flat_data)
		const tires = sub_data['$tires']
		if(tires){
			const all_tires = []
			const tire_signals = {
				"alarm_ok": [],
				"sensor_mute": [],
				"sensor_defective": [],
				// "sensor_loose": [],
				// "sensor_turned": [],
				// "sensor_battery_warning": [],
				// "sensor_over_temperature": [],
				"tire_leak": [],
				"temperature_warning": [],
				"over_inflation_critical": [],
				"over_inflation_warning": [],
				"under_inflation_warning": [],
				"under_inflation_critical": [],
			}
			for (const axle in tires){
				let axle_data = tires[axle]
				for (const tire in axle_data){
					let tire_data = axle_data[tire]
					let tire_ok = true
					tire_data.axle_index = axle.replace("axle_", "")
					tire_data.axle = parseInt(tire_data.axle_index) + 1
					tire_data.tire = tire.replace("tire_", "")
					all_tires.push(tire_data)
					switch (tire_data.sensor_status) {
						case 0:
							tire_ok = false
							tire_signals['sensor_mute'].push(tire_data)
							break
						case 2:
							tire_ok = false
							tire_signals['sensor_defective'].push(tire_data)
							break
					}
					if (tire_data.air_leak_status == 1) {
						tire_ok = false
						tire_signals['tire_leak'].push(tire_data)
					}
					if (tire_data.temperature_state == 1){
						tire_ok = false
						tire_signals['temperature_warning'].push(tire_data)
					}
					switch (tire_data.pressure_state){
						case 0:
							tire_ok = false
							tire_signals['over_inflation_critical'].push(tire_data)
							break
						case 1:
							tire_ok = false
							tire_signals['over_inflation_warning'].push(tire_data)
							break
						case 3:
							tire_ok = false
							tire_signals['under_inflation_warning'].push(tire_data)
							break
						case 4:
							tire_ok = false
							tire_signals['under_inflation_critical'].push(tire_data)
							break
					}
					if (tire_ok){
						tire_signals['alarm_ok'].push(tire_data)
					}
				}
			}
			let tires_update = {}
			for(const signal in tire_signals){
				let tires = tire_signals[signal]
				if (tires.length){
					tires_update[`$tpms.${signal}`] = tires
					signals.push(`@tpms.${signal}`)
				}
			}
			// tires_data
			clear_object(tires_dotted)
			Object.assign(tires_dotted, dotted.dot(tires_update))
			// console.log(tires_dotted)
			Object.assign(syrusjs_variables, tires_dotted);
		}
		fireInternalSignal(...signals)
	}
	function onSleepOn() {
		fireInternalSignal(`@power_save.sleep`);
	}

	// TAIP ex
	// IB:$ibutton.connected.id,IS:$ibutton.last.id
	// IB:$ibutton.authorized.connected.id,IS:$ibutton.authorized.last.id
	// JSON ex
	// connected_ib:$ibutton.connected
	const ib_dotted = {}
	function onIbuttonChange(update){
		let ib_dotted_update = dotted.dot({ $ibutton: update })
		// nullify all keys on ib_dotted
		clear_object(ib_dotted)
		// apply new update
		Object.assign(ib_dotted, ib_dotted_update)
		Object.assign(syrusjs_variables, ib_dotted);
	}
	// TAIP ex
	// EA_A:$temp.aliases.sensor1.value
	// EA_A:$temp.sensor_list[0].value
	// JSON ex
	// temp:$temp.aliases
	const temp_dotted = {}
	function onTemperatureChange(update) {
		let temp_dotted_update = dotted.dot({ $temp: update })
		// nullify all keys on ib_dotted
		clear_object(temp_dotted)
		// apply update
		Object.assign(temp_dotted, temp_dotted_update);
		Object.assign(syrusjs_variables, temp_dotted);
	}

	function onSerialMessage(message){
		Object.assign(syrusjs_variables, {"$serial.incoming.message": message});
		fireInternalSignal(`@serial.incoming.message`);
	}

	const fatigue_dotted = {}
	function onFatigueEvent(update){
		// console.log('fatigue update', {update, fatigue_dotted});
		let { epoch, event, channel } = update
		if (!event) return
		let recent = (Date.now() / 1e3 - epoch) < 30
		if (!recent || fatigue_dotted['$fatigue_sensor.epoch'] == update.epoch) return
		let fatigue_dotted_update = dotted.dot({ $fatigue_sensor: update })
		// nullify keys on object
		clear_object(fatigue_dotted)
		// apply update
		Object.assign(fatigue_dotted, fatigue_dotted_update)
		Object.assign(syrusjs_variables, fatigue_dotted)
		// deternmine signals
		const signals = [
			`@${channel}.fatigue_sensor.event`,
			`@${channel}.fatigue_sensor.${event}`,
			`@fatigue_sensor.signal`,
			`@fatigue_sensor.${event}`,
		]
		if (fatigue_dotted['$fatigue_sensor.state'] != update.state) {
			signals.push(
				`@{channel}.fatigue_sensor.${update.state}`,
				`@fatigue_sensor.${update.state}`,
			)
		}
		// fire signals
		fireInternalSignal(...signals)
	}

	function onMDTMessage(update){
		Object.assign(syrusjs_variables, dotted.dot({ $mdt: update }))
		fireInternalSignal(`@mdt.message`);
	}

	// Establishing all the subcriptions to redis throught the SDK to get the changes on the S4.
	GPS.watchPosition(onGpsData, console.error);
	IOS.watchInputState("*", onInputChange, console.error);
	ECU.watchECUParams(onEcuParamsChange, console.error);
	Geofences.watchGeofences(onGeofenceChange, console.error, { namespace: configuration.namespace });
	Geofences.watchGroups(onGeofenceGroupChange, console.error, { namespace: configuration.namespace });
	Network.onNetworkChange(onNetworkChange, console.error);
	Accelerometer.onMotionChange(onMotionChange, console.error);
	Accelerometer.on(onMotionEvent, console.error);
	System.onSleepOn(onSleepOn, console.error);
	IButton.onIButtonChange(onIbuttonChange, console.error)
	Temperature.onTemperatureChange(onTemperatureChange, console.error)
	Serial.onIncomingMessage(onSerialMessage, console.error)
	Serial.onMDTMessage(onMDTMessage, console.error)
	Fatigue.onFatigueEvent(onFatigueEvent, console.error)
	Bluetooth.onBluetoothUpdate(
		(channel, payload) => {
			// console.log('bt.watcher', {channel, payload})
			// backwards compatability
			if (channel.endsWith('/CALL_BUTTON')){
				fireInternalSignal(
					"@bluetooth.button.signal",
					`@bluetooth.button.call_${payload}x`,
				)
				return
			}
			if (channel.endsWith('/action')){
				let { button, action, action_multiplier } = payload
				if (action != 'tap') return
				fireInternalSignal(
					"@bluetooth.button.signal",
					`@bluetooth.button.${button}_${action_multiplier}x`,
				)
				return
			}
		},
		console.error
	)

	// Get the current state of network, ios and ecu, to update the signals and variables
	Network.getNetworks().then(onNetworkChange).catch(console.error);
	Geofences.getAll({ namespace: configuration.namespace }).then(onGeofenceChange).catch(console.error);
	IOS.getAll().then(onInputChange).catch(console.error);
	ECU.getECUParams().then(onEcuParamsChange).catch(console.error);

	/**
	 * All the config file are watch in case of any change to restart the app a get the new configuration
	 * TODO: pending improve  get the new config without a restart.
	 */
	var { destinationsPath, configPath, schemaPath } = configuration;
	fs.watch(schemaPath, () => {
		console.log(`detected changes on file ${schemaPath}, restarting app`);
		process.kill(process.pid, "SIGTERM");
	});
	fs.watch(destinationsPath, () => {
		console.log(`detected changes on file ${destinationsPath}, restarting app`);
		process.kill(process.pid, "SIGTERM");
	});
	fs.watch(configPath, () => {
		console.log(`detected changes on file ${configPath}, restarting app`);
		process.kill(process.pid, "SIGTERM");
	});

	/* Arbitrary callback to check if syrus was restarted by psm.
	This is not called immediately since the unix time can be invalid while there is no network
	and it is established to be the internal clock of S4
	*/
	setTimeout(async () => {
		try{
			let wakeup = await System.getLastWakeUp()
			if (wakeup) {
				const newTimestamp = dayjs(wakeup.timestamp)
				let reason = wakeup.wakeup_reason.toLowerCase()
				if (reason == "mot") reason = "motion"
				if (reason == "elapsed_time") reason = "time"
				if (reason == "electrical_rst") reason = "electrical_reset"
				if (newTimestamp.unix() != syrusjs_variables["$power_save.time"]) {
					syrusjs_variables["$power_save.time"] = newTimestamp.unix()
					syrusjs_variables["$power_save.reason"] = wakeup.wakeup_reason
					fireInternalSignal(
						`@power_save.wakeup`,
						`@power_save.${reason}`,
					)
				}
			}
		} catch(err){
			console.error("ERROR getting wakeup reason")
			console.error(err)
		}
	}, 1000 * 15);

	// Interval for sync data from the utils within syrusjs
	// In case redis fails for some reason or the syrus-interface fails for any reason
	// Could be improved this handling this part inside the Syrus4-SDK
	setInterval(() => {
		onNetworkChange();
		IOS.getAll().then(onInputChange).catch(console.error);
		ECU.getECUParams().then(onEcuParamsChange).catch(console.error);
	}, 1000 * 60 * 5);

	// update $modem information periodically
	// TODO: update with publish
	setInterval(async ()=>{
		try {
			var modem = await System.modem()
			for (const key in modem) {
				syrusjs_variables[`$modem.${key.toLowerCase()}`] = modem[key]
			}
		} catch (error) {
			console.error(error)
		}
	}, 1000 * 60 * 1)
};
