import fs from "fs"
import { redisClient } from "syrus4-nodejs/dist/Redis.js"
import { app_data_folder } from "../scripts/globals.js"
/**
 * Queue is buffer handler intended to storage a lot of events inside files, the idea is keep a easy implementation while api is the most compatible with javascript arrays
 * Right now class only implements sync methods to no break the usage where other are using array as storage
 * but could be extended to use async methods for get, caching the length, and user WritableStream and ReadableStreams for operate with files and improve memory usage even more.
 */
export class Queue {
	constructor(name, opts) {
		let _default = { dataType: "string", mode: "fifo", folder: app_data_folder, memoryInstance: redisClient, chunkSize: 1000 };
		opts = Object.assign(_default, opts);
		this.memoryInstance = opts.memoryInstance;
		this.name = name;
		this.opts = opts;
		this.folder = `${opts.folder}/.${name}`;
		this.promiseReady = new Promise(resolver => {
			this.resolver = resolver;
		});
		if (!fs.existsSync(this.folder)) {
			// Previos version of queue system creates a folder with same system creating to files incoming and outgoing all of this using the npm package
			// https://github.com/behrad/mqtt-nedb-store this breaks the compatibility with this new Queue.js
			// if the file exists must be removed and recreated
			if(fs.existsSync(`${this.folder}/incoming`) || fs.existsSync(`${this.folder}/outgoing`)){
				fs.readdirSync(this.folder).forEach(file => {
					fs.unlinkSync(`${this.folder}/${file}`);
				});
			}
			fs.mkdirSync(this.folder);
		}
		this.list = [];
		this.init();
	}

	async clear() {
		let prepend = this.opts.folder;
		let key = this.name;
		this.list = [];
		fs.readdirSync(this.folder).forEach(file => {
			fs.unlinkSync(`${this.folder}/${file}`);
		});
		await this.memoryInstance.del(`${prepend}_${key}`);
	}

	destroy() {
		this.clear();
		fs.unlinkSync(`${this.folder}`);
	}

	ready() {
		return this.promiseReady;
	}

	init() {
		let prepend = this.opts.folder;
		let key = this.name;
		this.opts.memoryInstance
			.lrange(`${prepend}_${key}`, 0, -1)
			.then(result => {
				this.list = result.reverse().map(txt => {
					if (this.opts.dataType == "json") return JSON.parse(txt);
					return txt;
				});
				this.resolver(this.list);
			})
			.catch(err => {
				console.error("Error in redis for Queuejs", err);
			});
	}

	get(index) {
		let chunkSize = this.opts.chunkSize;
		let length = this.length;
		if (index < 0) return undefined;
		if (index >= length) return undefined;
		if (index <= this.list.length){
			if (this.opts.dataType == "json") return JSON.parse(this.list[index]);
			else return this.list[index];
		}
		let inFile = Math.ceil((index - this.list.length) / chunkSize) - 1;
		let files = this.__getFiles();

		let file = files[inFile];
		const data = fs.readFileSync(file, "UTF-8");
		const lines = data.split(/\r?\n/);
		lines.pop();
		let position = (index - this.list.length) % chunkSize;
		if (this.opts.dataType == "json") return JSON.parse(lines[position]);
		else return lines[position];
	}

	set(index, data) {
		// TODO: Implement this
	}

	slice(start, end){
		// TODO: implement this
	}

	range(start, end){
		return this.slice(start,end)
	}

	get length() {
		var length = this.list.length;
		for (const file of this.__getFiles()) {
			length += this.__getFileLength(file);
		}
		return length;
	}

	push(...elements) {
		let prepend = this.opts.folder;
		let key = this.name;
		for (const el of elements) {
			let txt = el
			if (this.opts.dataType == "json") {
				txt = JSON.stringify(el);
			}
			if (this.list.length < this.opts.chunkSize && this.__getFiles().length == 0) {
				this.list.push(el);
				this.opts.memoryInstance.lpush(`${prepend}_${key}`, txt);
			} else {
				this.__appendToLastFile(txt);
			}
		}
	}

	shift() {
		let prepend = this.opts.folder;
		let key = this.name;
		if (this.list.length == 0) {
			return;
		}
		this.opts.memoryInstance.rpop(`${prepend}_${key}`);
		var el = this.list.shift();
		if (this.list.length == 0) {
			this.__passFileToMemory();
		}
		return el;
	}

	unshift(...elements){
		let prepend = this.opts.folder;
		let key = this.name;
		for (const el of elements) {
			let txt = el
			if (this.opts.dataType == "json") {
				txt = JSON.stringify(el);
			}
			this.list.unshift(txt)
			this.opts.memoryInstance.rpush(`${prepend}_${key}`, txt);
		}
	}

	pop(){
		let prepend = this.opts.folder;
		let key = this.name;
		if (this.__getFiles().length == 0) {
			var el = this.list.pop()
			this.opts.memoryInstance.lpop(`${prepend}_${key}`);
			if (this.list.length == 0) {
				this.__passFileToMemory();
			}
			return el;
		} else {
			let file = this.__getFiles().pop();
			let lines = fs.readFileSync(file, { encoding: "utf-8" }).split(/\r?\n/);
			lines.pop();
			let line = lines.pop();
			fs.writeFileSync(file, lines.join(/\n/g) + "\n", { encoding: "utf-8"});
			if (this.opts.dataType == "json") {
				return JSON.parse(line);
			} else {
				return line;
			}
		}
	}

	get first() {
		return this.list[0];
	}

	get 0() {
		return this.first;
	}

	get last() {
		if (this.list.length < this.opts.chunkSize && this.__getFiles().length == 0) {
			return this.list[this.list.length - 1];
		} else {
			let file = this.__getFiles().pop();
			let line = fs.readFileSync(file, { encoding: "utf-8" }).split(/\r?\n/);
			line = line[line.length - 2];
			if (this.opts.dataType == "json") {
				return JSON.parse(line);
			} else {
				return line;
			}
		}
	}

	__getFiles() {
		return fs
			.readdirSync(this.folder)
			.sort((a, b) => a.localeCompare(b))
			.map(f => {
				return `${this.folder}/${f}`;
			});
	}

	__getFileLength(file) {
		// read contents of the file
		const data = fs.readFileSync(file, "UTF-8");
		// split the contents by new line
		const lines = data.split(/\r?\n/);
		return lines.length - 1;
	}

	__appendToLastFile(data) {
		var files = this.__getFiles();
		if (files.length == 0) {
			files = [this.__createNewFile(0)];
		}
		let file = files.pop();
		if (this.__getFileLength(file) >= this.opts.chunkSize) {
			file = this.__createNewFile(files.length + 1);
		}
		if (typeof data == "string") fs.appendFileSync(file, data + "\n");
		else fs.appendFileSync(file, JSON.stringify(data) + "\n");
	}

	__createNewFile(index) {
		fs.writeFileSync(`${this.folder}/${index}`, "");
		return `${this.folder}/${index}`;
	}

	__passFileToMemory() {
		var files = this.__getFiles();
		if (files.length == 0) {
			return false;
		}
		let file = files[0];
		let prepend = this.opts.folder;
		let key = this.name;
		var lines = fs.readFileSync(file, { encoding: "utf-8" }).split(/\r?\n/);
		for (const line of lines) {
			if (line) {
				if (this.opts.dataType == "json") var data = JSON.parse(line);
				else data = line;
				this.list.push(data);
				this.opts.memoryInstance.lpush(`${prepend}_${key}`, line);
			}
		}
		fs.unlinkSync(file);
		this.__renameFiles();
	}

	__renameFiles() {
		for (const file of this.__getFiles()) {
			fs.renameSync(file, `${this.folder}/${parseInt(file.split("/").pop()) - 1}`);
		}
	}
}

// TODO: implement different mode: lifo
// TODO: make push, shift , unshift, length, first, last function async and  non-blocking
